<?php

namespace EventPortal\Http\Controllers\Delegate\Account;

use EventPortal\Domain\Event\ValueObjects\PageHeader;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Util\DelegateRender;
use EventPortal\Domain\Util\Notification\Facades\Notification;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class ChangePasswordController extends Controller
{
    public function edit(Request $request): Response
    {
        $form = Form::make('delegate.account.password.edit')->model($request->user())->render();

        $rules = collect(config('auth-password.validation.password'))
            ->reject(static function ($rule) {
                return is_string($rule);
            })
            ->map(static function ($rule) {
                return [
                    'label' => $rule->frontEndLabel(),
                    'validation' => $rule->frontEndValidation(),
                ];
            })
            ->values();

        return DelegateRender::component('Delegate/Account/Password/Edit')
            ->withPageHeader(
                PageHeader::make()
                    ->title("Hi, {$request->user()->full_name}! Welcome to your account.")
                    ->description('Change your password.')
            )
            ->withForm($form)
            ->withData([
                'rules' => $rules,
            ])
            ->get();
    }

    public function update(Request $request): RedirectResponse
    {
        Form::make('delegate.account.password.edit')
            ->model($request->user())
            ->validate($request->all())
            ->handle($request->all());

        Notification::make('delegate.account.password.updated')
            ->to($request->user())
            ->send();

        toast()->error(trans('delegate.password.updated'));

        return redirect()->route('account.password.edit');
    }
}
