<?php

namespace EventPortal\Tests\Feature\Delegate\Event;

use Carbon\Carbon;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Tests\Support\Factories\AgendaFactory;
use EventPortal\Tests\Support\Factories\PathwayFactory;
use EventPortal\Tests\Support\Factories\SessionFactory;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class AgendaIndexTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test */
    public function view_agenda_page()
    {
        $this->withoutExceptionHandling();
        Carbon::setTestNow(Carbon::parse('21 October 2020'));

        $delegate = factory(Delegate::class)->state('snapshot')->create();
        $event = factory(Event::class)->state('snapshot')->create();
        $pathway = factory(Pathway::class)->state('default')->create([
            'event_id' => $event->id,
        ]);

        factory(EventDelegate::class)->create([
            'event_id' => $event->id,
            'delegate_id' => $delegate->id,
            'pathway_id' => $pathway->id,
        ]);



        AgendaFactory::make(now(), $event)
            ->createSession($pathway, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'One',
                    ])
                    ->createVenue([
                        'name' => 'Venue one',
                    ]);
            });

        $this->assertMatchesJsonSnapshot(
            $this->actingAs($delegate->user)->get(route('agenda.index', $event))->props()
        );
    }

    /** @test */
    public function view_agenda_in_date_order()
    {
        Carbon::setTestNow(Carbon::parse('21 October 2020'));

        $delegate = factory(Delegate::class)->state('snapshot')->create();
        $event = factory(Event::class)->state('snapshot')->create();
        $pathway = factory(Pathway::class)->state('default')->create([
            'event_id' => $event->id,
        ]);

        factory(EventDelegate::class)->create([
            'event_id' => $event->id,
            'delegate_id' => $delegate->id,
            'pathway_id' => $pathway->id,
        ]);

        AgendaFactory::make(now()->addDay(), $event)
            ->createSession($pathway, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'One',
                    ])
                    ->createVenue([
                        'name' => 'Venue one',
                    ]);
            });

        AgendaFactory::make(now(), $event)
            ->createSession($pathway, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'One',
                    ])
                    ->createVenue([
                        'name' => 'Venue one',
                    ]);
            });

        $this->assertMatchesJsonSnapshot(
            $this->actingAs($delegate->user)->get(route('agenda.index', $event))->props()
        );
    }
}
