<?php

namespace EventPortal\Tests\Unit\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Actions\UpdateSections;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionOption;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Form\ValueObjects\SectionData;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Collection;

class UpdateSectionsTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_update_form_section(): void
    {
        $step = factory(FormStep::class)->create();
        factory(FormStepSection::class)->create(['form_step_id' => $step->id]);

        (new UpdateSections)->execute($this->getUpdatedSection(), $step);

        $section = $step->sections->first();
        $this->assertEquals('Updated Test Section', $section->name);
        $this->assertEquals('Updated test uuid', $section->uuid);
        $this->assertEquals(1, $section->order);
    }

    private function getUpdatedSection(array $override = []): Collection
    {
        return collect(array_merge([
            [
                'id' => 1,
                'name' => 'Updated Test Section',
                'order' => 1,
                'uuid' => 'Updated test uuid',
            ],
        ], $override))->mapInto(SectionData::class);
    }

    /** @test **/
    public function can_add_new_form_section(): void
    {
        $step = factory(FormStep::class)->create();
        factory(FormStepSection::class)->create(['form_step_id' => $step->id]);

        (new UpdateSections)->execute($this->getNewSection(), $step);

        $section = $step->sections->first();
        $this->assertEquals('New test section', $section->name);
        $this->assertEquals('New test uuid', $section->uuid);
        $this->assertEquals(1, $section->order);
    }

    private function getNewSection(array $override = []): Collection
    {
        return collect(array_merge([
            [
                'name' => 'New test section',
                'order' => 1,
                'uuid' => 'New test uuid',
            ],
        ], $override))->mapInto(SectionData::class);
    }

    /** @test **/
    public function deletes_unused_form_sections(): void
    {
        $step = factory(FormStep::class)->create();
        factory(FormStepSection::class)->create(['form_step_id' => $step->id]);

        (new UpdateSections)->execute(collect([]), $step);

        $this->assertCount(0, $step->sections);
    }

    /** @test **/
    public function deletes_questions_options_validations_when_deleting_section(): void
    {
        $step = factory(FormStep::class)->create();
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)->create(['form_step_section_id' => $section->id]);
        factory(QuestionOption::class)->create(['question_id' => $question->id]);
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        (new UpdateSections)->execute(collect([]), $step);

        $this->assertCount(0, Question::all());
        $this->assertCount(0, QuestionOption::all());
        $this->assertCount(0, QuestionValidation::all());
    }
}
