<?php

namespace EventPortal\Tests\Unit\Util;

use Carbon\Carbon;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\ValueObjects\EventDetails;
use EventPortal\Domain\Event\ValueObjects\PageHeader;
use EventPortal\Domain\Util\DelegateRender;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class DelegateRenderTest extends TestCase
{
    use MatchesSnapshots, RefreshDatabase;

    /** @test **/
    public function can_add_event_details()
    {
        $event = factory(Event::class)->create(['name' => 'Event Name']);
        factory(Agenda::class)->create([
            'event_id' => $event->id,
            'date' => Carbon::parse('2020-05-20'),
        ]);

        $render = DelegateRender::component('Test\Component')
            ->withEventDetails(EventDetails::make($event))
            ->getData('eventDetails');

        $this->assertArraySubset([
            'name' => 'Event Name',
            'date' => '20/05/2020',
        ], $render);
    }

    /** @test **/
    public function can_add_title()
    {
        $render = DelegateRender::component('Test\Component')
            ->withTitle('Page Title')
            ->getData('pageHeader');

        $this->assertEquals([
            'title' => 'Page Title',
        ], $render);
    }

    /** @test **/
    public function can_add_page_header()
    {
        $render = DelegateRender::component('Test\Component')
            ->withPageHeader(PageHeader::make()
                ->description('Description')
                ->title('Page Title'))
            ->getData('pageHeader');

        $this->assertEquals([
            'title' => 'Page Title',
            'description' => 'Description',
            'callToAction' => null,
        ], $render);
    }
}
